@@ -26,8 +26,6 @@
 #define STR_METHOD install("method")
 #define STR_NALAST install("na.last")
 #define STR_DECREA install("decreasing")
-#define STR_QUICK  mkString("quick")
-#define STR_SHELL  mkString("shell")
 #define STR_FACTOR mkChar("factor")
 
 static int K = 8;
@@ -291,7 +289,7 @@ static inline int getIndex(SEXP ptr, const SEXP *restrict cmp, int *lkpTbl) {
   }
 }
 
-static SEXP callToSort (SEXP x, SEXP method, SEXP env) {
+static SEXP callToSort (SEXP x, const char* method, SEXP env) {
   SEXP call = PROTECT(allocVector(LANGSXP, 4));
   SETCAR(call, STR_SORT);
   
@@ -300,7 +298,7 @@ static SEXP callToSort (SEXP x, SEXP method, SEXP env) {
   SET_TAG(s, install("x"));
   
   s = CDR(s);
-  SETCAR(s, method);
+  SETCAR(s, PROTECT(mkString(method)));
   SET_TAG(s, STR_METHOD);
   
   s = CDR(s);
@@ -308,11 +306,11 @@ static SEXP callToSort (SEXP x, SEXP method, SEXP env) {
   SET_TAG(s, STR_NALAST);
   
   SEXP out = PROTECT(eval(call, env));
-  UNPROTECT(2);
+  UNPROTECT(3);
   return out;
 }
 
-static SEXP callToSort2 (SEXP x, SEXP method, const int desc, const int na, SEXP env) {
+static SEXP callToSort2 (SEXP x, const char* method, const int desc, const int na, SEXP env) {
   SEXP call = PROTECT(allocVector(LANGSXP, 5));
   SETCAR(call, STR_SORT);
   
@@ -321,7 +319,7 @@ static SEXP callToSort2 (SEXP x, SEXP method, const int desc, const int na, SEXP
   SET_TAG(s, install("x"));
   
   s = CDR(s);
-  SETCAR(s, method);
+  SETCAR(s, PROTECT(mkString(method)));
   SET_TAG(s, STR_METHOD);
   
   s = CDR(s);
@@ -333,11 +331,11 @@ static SEXP callToSort2 (SEXP x, SEXP method, const int desc, const int na, SEXP
   SET_TAG(s, STR_DECREA);
   
   SEXP out = PROTECT(eval(call, env));
-  UNPROTECT(2);
+  UNPROTECT(3);
   return out;
 }
 
-static SEXP callToOrder (SEXP x, SEXP method, bool desc, Rboolean na, SEXP env) { // # nocov start
+SEXP callToOrder (SEXP x, const char* method, bool desc, Rboolean na, SEXP env) {
   SEXP call = PROTECT(allocVector(LANGSXP, 5));
   SETCAR(call, STR_ORDER);
   
@@ -346,7 +344,7 @@ static SEXP callToOrder (SEXP x, SEXP method, bool desc, Rboolean na, SEXP env)
   SET_TAG(s, install("..."));
   
   s = CDR(s);
-  SETCAR(s, method);
+  SETCAR(s, PROTECT(mkString(method)));
   SET_TAG(s, STR_METHOD);
   
   s = CDR(s);
@@ -358,9 +356,9 @@ static SEXP callToOrder (SEXP x, SEXP method, bool desc, Rboolean na, SEXP env)
   SET_TAG(s, STR_DECREA);
   
   SEXP out = PROTECT(eval(call, env));
-  UNPROTECT(2);
+  UNPROTECT(3);
   return out;
-} // # nocov end
+}
 
 /*
  *  Character sorting
@@ -395,11 +393,11 @@ SEXP cpsortR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env, SEXP
   
   const int early = xlen == n;
   SEXP valSorted = early ? (
-    cindex ? PROTECT(callToOrder(uVals, STR_SHELL, dcr, na_pos, env)) : 
+    cindex ? PROTECT(callToOrder(uVals, "shell", dcr, na_pos, env)) : 
              (cl ? (dcr ? PROTECT(rsortrev(uVals)) : PROTECT(rsort(uVals))) :
-                   PROTECT(callToSort2(uVals, STR_QUICK, dcr, na_pos, env)))
+                   PROTECT(callToSort2(uVals, "quick", dcr, na_pos, env)))
   ) : ( cl ? PROTECT(rsort(uVals)) : 
-             PROTECT(callToSort(uVals, STR_QUICK, env)));
+             PROTECT(callToSort(uVals, "quick", env)));
 
   /*if (early && cindex) {
     UNPROTECT(2);
@@ -545,11 +543,14 @@ SEXP cpsortR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env, SEXP
  *  Character to factor conversion
  */
 
-SEXP charToFactR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env) {
+SEXP charToFactR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env, SEXP addNA) {
   
   if (!IS_BOOL(decreasing)) {
     error("Argument 'decreasing' must be TRUE or FALSE.");
   }
+  if (!IS_BOOL(addNA)) {
+    error("Argument 'addNA' must be TRUE or FALSE.");
+  }
   /*if (!IS_LOGICAL(nalast)) {
     error("Argument 'na.last' must be TRUE, FALSE or NA.");
   }*/
@@ -562,12 +563,13 @@ SEXP charToFactR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env)
 
   const int na_pos = asLogical(nalast);
   const int dcr = asLogical(decreasing);
+  const int addNAv = asLogical(addNA);
   const int xlen = LENGTH(x);
   
   SEXP uVals = PROTECT(dupVecSort(x));
   const int n = LENGTH(uVals);
   
-  SEXP valSorted = PROTECT(callToSort2(uVals, STR_QUICK, dcr, 1, env));
+  SEXP valSorted = PROTECT(callToSort2(uVals, "quick", dcr, 1, env));
   SEXP *restrict pvalSorted = STRING_PTR(valSorted);
   
   int NAidx = -1;
@@ -596,9 +598,16 @@ SEXP charToFactR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env)
   SEXP ans = PROTECT(allocVector(INTSXP, xlen));
   int *restrict pans = INTEGER(ans);
   
-  OMP_PARALLEL_FOR(nth)
-  for (int j=0; j<xlen; ++j) {
-    pans[j] = LOOKUP_VAL(px[j]) + 1;
+  if (addNAv == 0) {
+    OMP_PARALLEL_FOR(nth)
+    for (int j=0; j<xlen; ++j) {
+      pans[j] = px[j] == NA_STRING ? NA_INTEGER : LOOKUP_VAL(px[j]) + 1;
+    }
+  } else {
+    OMP_PARALLEL_FOR(nth)
+    for (int j=0; j<xlen; ++j) {
+      pans[j] = LOOKUP_VAL(px[j]) + 1;
+    }
   }
 
   if (na_pos == NA_LOGICAL) {
@@ -616,6 +625,9 @@ SEXP charToFactR (SEXP x, SEXP decreasing, SEXP nthread, SEXP nalast, SEXP env)
   }
   
   free(lookupTable);
+  if (addNAv == 0) {
+    SETLENGTH(valSorted, LENGTH(valSorted)-1);
+  }
   setAttrib(ans, R_LevelsSymbol, valSorted);
   SEXP classV = PROTECT(allocVector(STRSXP,1));
   SET_STRING_ELT(classV, 0, STR_FACTOR);
